-------------------------------------------------------------------------------
-- Title      : calcul des initialisations ROM 128x8
-- Project    : 
-------------------------------------------------------------------------------
-- File       : texttorom.vhd
-- Author     :  http://vhdl33.free.fr 
-- Company    : 
-- Last update: 2008/11/13
-- Platform   : 
-------------------------------------------------------------------------------
-- Description: 
-------------------------------------------------------------------------------
-- Revisions  :
-- Date        Version  Author  Description
-- 2006/10/11  1.0      nouel	Created
-------------------------------------------------------------------------------

LIBRARY std;
USE std.textio.ALL;
LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE ieee.numeric_std.ALL;

ENTITY texttorom IS
  GENERIC (nom_fichier_entree : string := "messages.txt";
           nom_fichier_sortie : string := "rom_init.vhd");
END texttorom;


ARCHITECTURE ecriture OF texttorom IS 
  
  
  
BEGIN  -- ecriture

  ecr  : PROCESS
    FILE fichier1 : text IS IN nom_fichier_entree;
    FILE fichier2 : text IS OUT nom_fichier_sortie;
    VARIABLE i  : natural RANGE 0 TO 128; 
    TYPE t_128x8 IS ARRAY (0 TO 127) OF std_logic_vector(7 DOWNTO 0);
    VARIABLE table_messages : t_128x8 := (OTHERS => "00000000");
    VARIABLE ligne : line;
    VARIABLE c : character;
    VARIABLE bon : boolean ;
    VARIABLE cd : natural;
    TYPE t_8x128 IS ARRAY (0 TO 7) OF std_logic_vector(127 DOWNTO 0);
    VARIABLE table_init : t_8x128 :=
      (OTHERS =>X"00000000000000000000000000000000");
    VARIABLE bit128 : std_logic_vector(127 DOWNTO 0);
    VARIABLE bit8 : std_logic_vector(7 DOWNTO 0);
    VARIABLE bit4 : std_logic_vector(3 DOWNTO 0);
    CONSTANT en_tete : string := "--*  ENSEIRB: Fichier d'initialisation  d'une ROM128x8 ***";
    VARIABLE quartet : natural RANGE 0 TO 15;
    VARIABLE hexa : character := '0';
    CONSTANT vhdl1 : string  := "TYPE t_bitvec IS ARRAY (0 TO 7) OF bit_vector(127 DOWNTO 0);";
    CONSTANT vhdl2 : string  := "CONSTANT init_rom : t_bitvec := (";
    
  BEGIN 
    ASSERT false REPORT "Debut du processus de conversion" SEVERITY note;
    i := 0;                             -- debut de table ascii
    l1: WHILE NOT ENDFILE(fichier1) LOOP
      readline(fichier1, ligne);
      l2:  LOOP
        read(ligne,c, bon);               -- lecture d'un caractere
        EXIT l2 WHEN NOT bon;     
          IF i < 128 THEN
            -- extraction du code ascii et mise dans la table
            table_messages(i) := std_logic_vector(to_unsigned(character'pos(c),8));  
            i := i+1;
          ELSE
            ASSERT false  REPORT "La memoire ROM est pleine"
              SEVERITY error;
            EXIT l1;
          END IF;
        WAIT FOR 1 ns;                   -- juste pour observer
      END LOOP l2;
      IF i < 128 THEN 
        table_messages(i) := (OTHERS => '0');  -- le 0 de fin de message
         i:= i+1;
      ELSE
        ASSERT false  REPORT "La memoire ROM est pleine"
          SEVERITY error;
        EXIT l1;
      END IF;
    END LOOP l1;
    -- conversion de la table 128x8 en une table 8x128
    -- conversion bit par bit
    l3: FOR j IN 0 TO 7 LOOP
      l4:   FOR k IN 0 TO 127 LOOP
        bit8 := table_messages(k);
        bit128(k) := bit8(j);
  --      WAIT FOR 1 ns;                 -- juste pour observer
      END LOOP l4;  -- k
      table_init(j) := bit128;
    END LOOP l3;  -- j
    -- ecriture dans le fichier final
    -- sous la forme
--     -- TYPE t_bitvec IS ARRAY (0 TO 7) OF bit_vector(127 DOWNTO 0);
--   CONSTANT init_rom : t_bitvec := (
--     X"00000010000000000000000000000101",
--     X"00000010000000040000000000000101",
--     X"00000010000000000000000000000101",
--     X"00000010000000000000000000000101",
--     X"00000010000000000000050000000101",
--     X"00000010000000000000000000000101",
--     X"00000010000000000000000000000101",
--     X"00000010000000000000000000000101");
    ASSERT false REPORT "Ecriture fichier de sortie"
      SEVERITY note;
    write(ligne,en_tete);
    writeline(fichier2,ligne);
    write(ligne,cr);
    writeline(fichier2,ligne);
    write(ligne,vhdl1);
    writeline(fichier2,ligne);
    write(ligne,vhdl2);
    writeline(fichier2,ligne); 
    l5: FOR i IN 0 TO 7 LOOP
      write(ligne, 'X');  write(ligne, '"');
      bit128 := table_init(i);
      l6: FOR j IN 31 DOWNTO 0 LOOP
        bit4 := bit128(4*j + 3 DOWNTO 4*j);
        quartet := to_integer(unsigned(bit4));
        IF quartet > 9 THEN             -- traitement A =>  F
          hexa := character'val(55 + quartet);
        ELSE                            -- traitement 0 => 9
          hexa := character'val(48 + quartet); 
        END IF;
        write(ligne, hexa);
      END LOOP;  -- j
      write(ligne, '"');
      IF i = 7 THEN
        write(ligne, ')'); write(ligne, ';');    
      ELSE
        write(ligne, ',');
      END IF;
      writeline(fichier2,ligne);        
    END LOOP;  -- i
    ASSERT false REPORT "Fin de generation des donnees d'initialisation";
    WAIT;
  END PROCESS;


END;
