-------------------------------------------------------------------------------
-- Title      : Interface LCD utilisant Picoblaze
-- Project    : 
-------------------------------------------------------------------------------
-- File       : topdesign.vhd
-- Author     : http://vhdl33.free.fr
-- Company    : 
-- Last update: 2008/11/11
-- Platform   : 
-------------------------------------------------------------------------------
-- Description: 
-------------------------------------------------------------------------------
-- Revisions  :
-- Date        Version  Author  Description
-- 2006/07/18  1.0      nouel	Created
-------------------------------------------------------------------------------


LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;
USE ieee.numeric_std.ALL;
--
------------------------------------------------------------------------------------
--
--
ENTITY topdesign    IS
  PORT (
    reset : IN std_logic;     -- bouton reset
    clk : IN std_logic;      -- Horloge carte 50 MHz
    lcd_rs : OUT  std_ulogic;  -- command 0; data 1
    lcd_rw : OUT  std_ulogic;  -- read 1; write 0
    lcd_e : OUT  std_ulogic;  -- enable
    lcd : INOUT std_logic_vector(3 DOWNTO 0);  -- lcd en lecture/ecriture
    strataflash_oe : OUT std_ulogic;  -- signaux flash
    strataflash_we : OUT std_ulogic;  -- en paralelle
    strataflash_ce : OUT std_ulogic);  -- sur lcd
END topdesign;
--
-------------------------------------------------------------------------------
ARCHITECTURE simple OF topdesign IS

  
  component embedded_kcpsm3 is
                              Port (      port_id : out std_logic_vector(7 downto 0);
                                          write_strobe : out std_logic;
                                          read_strobe : out std_logic;
                                          out_port : out std_logic_vector(7 downto 0);
                                          in_port : in std_logic_vector(7 downto 0);
                                          interrupt : in std_logic;
                                          interrupt_ack : out std_logic;
                                          reset : in std_logic;
                                          clk : in std_logic);
  end component;

-------------------------------------------------------------------------------

-- Signals used to connect embedded_KCPSM3
--
  SIGNAL port_id :  std_logic_vector(7 DOWNTO 0);
  SIGNAL write_strobe : std_ulogic;
  SIGNAL read_strobe : std_ulogic;
  SIGNAL out_port : std_logic_vector(7 downto 0);
  SIGNAL in_port :  std_logic_vector(7 downto 0);
  SIGNAL interrupt : std_ulogic;
  signal interrupt_ack : std_ulogic;
  SIGNAL lcd_in : std_logic_vector(3 DOWNTO 0);
  SIGNAL lcd_out : std_logic_vector(3 DOWNTO 0);
  SIGNAL lcd_reg : std_logic_vector(7 DOWNTO 0);
  SIGNAL lcd_drive : std_ulogic;

--------------------------------------------------------------------------------
BEGIN

  -- eviter les conflits entre lcd et flash
  strataflash_oe <= '1';
  strataflash_we <= '1';  
  strataflash_ce <= '1';

  -- picoblaze avec rom
  p1 : embedded_KCPSM3 PORT MAP (
    port_id       => port_id,
    write_strobe  => write_strobe,
    read_strobe   => read_strobe,
    out_port      => out_port,
    in_port       => in_port,
    interrupt     => interrupt,
    interrupt_ack => interrupt_ack,
    reset         => reset,
    clk           => clk);
  
-- decouplage des entrees/sortie bidirectionnelles

  in_port <= "0000" & lcd;

  lcd_e <= lcd_reg(0);
  lcd_rw <= lcd_reg(1);
  lcd_rs <= lcd_reg(2);
  lcd_drive <= lcd_reg(3);
  lcd <= lcd_reg(7 DOWNTO 4) WHEN lcd_drive = '1' ELSE "ZZZZ";  

  -- registre de sortie
  -- comme il n'y en a qu'un, l'adresse est pour l'instant ignoree
  regout: PROCESS
  BEGIN  -- PROCESS regin
    WAIT UNTIL rising_edge(clk);
    IF reset = '1'  THEN
      lcd_reg <= (OTHERS => '0');
    ELSIF write_strobe = '1' THEN
      lcd_reg <= out_port;
    END IF;
  END PROCESS regout;
  
-- decodage d'adresse
  -- pour l'instant ignore (n'importe quelle adresse fera l'affaire)
  

END simple;



